package net.wanji.utc.config.security;

import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.web.bind.annotation.GetMapping;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.contexts.SecurityContext;
import springfox.documentation.spring.web.plugins.Docket;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * @Description: swagger2配置，集成oauth2鉴权，默认扫描ApiOperation注解
 * @Author mapabc
 * @Date 2020/4/15 16:11
 */

public class IdentitySwaggerConfig {

    @Value("${security.oauth2.client.access-token-uri:http://localhost:15040/oauth/token}")
    private String accessTokenUri;

    @Resource
    @Lazy
    OAuth2AccessTokenRestTemplate restTemplate;

    @Bean
    public Docket createRestApi() {
        return new Docket(DocumentationType.SWAGGER_2)
                .apiInfo(apiInfo())
                .select()
                //.apis(RequestHandlerSelectors.withMethodAnnotation(ApiOperation.class))
                .apis(RequestHandlerSelectors.any())
                .paths(PathSelectors.any())
                .build()
                .securitySchemes(Arrays.asList(securityHeaderScheme()))
                .securityContexts(securityContexts())
                ;

    }

    /**
     * @return java.util.List<springfox.documentation.service.ApiKey>
     * @Description: 配置全局鉴权header
     * @Author mapabc
     * @Date 2020/4/16 11:35
     **/
    private SecurityScheme securityHeaderScheme() {
        return new ApiKey("Authorization", "tokenId", "header");
    }

    /**
     * @return springfox.documentation.service.OAuth
     * @Description: 鉴权配置，todo 获取的token自动在请求使用时加进去
     * @Author mapabc
     * @Date 2020/4/16 11:38
     **/
    private SecurityScheme securityAuthSchema() {
        return new OAuth(
                "Authorization",
                Collections.singletonList(new AuthorizationScope("all", "所有鉴权范围，认证服务器实际上不管这个")),
                Collections.singletonList(new ResourceOwnerPasswordCredentialsGrant(accessTokenUri))
        );
    }

    /**
     * @return java.util.List<springfox.documentation.spi.service.contexts.SecurityContext>
     * @Description: swagger 安全配置上下文
     * @Author mapabc
     * @Date 2020/4/16 11:34
     **/
    private List<SecurityContext> securityContexts() {
        List<SecurityContext> securityContexts = new ArrayList<>();
        securityContexts.add(SecurityContext.builder()
                .securityReferences(defaultAuth())
                .forPaths(PathSelectors.any())
                .build());
        return securityContexts;
    }

    /**
     * @return java.util.List<springfox.documentation.service.SecurityReference>
     * @Description: swagger 安全配置权限范围指定
     * @Author mapabc
     * @Date 2020/4/16 11:34
     **/
    private List<SecurityReference> defaultAuth() {
        AuthorizationScope authorizationScope = new AuthorizationScope("all", "全部访问权限");
        AuthorizationScope[] authorizationScopes = new AuthorizationScope[1];
        authorizationScopes[0] = authorizationScope;
        List<SecurityReference> securityReferences = new ArrayList<>();
        securityReferences.add(new SecurityReference("Authorization", authorizationScopes));
        return securityReferences;
    }

    /**
     * @return springfox.documentation.service.ApiInfo
     * @Description: swagger 描述展示信息配置
     * @Author mapabc
     * @Date 2020/4/16 11:33
     **/
    public ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("GIS云中台 API 接口服务")
                .description("GISC-Restful Open API 文档")
                .version("v1")
                .build();
    }

    @ApiOperation(value = "token获取", notes = "token获取", response = String.class)
    @GetMapping(value = "/getSwaggerToken")
    public String getSwaggerToken() throws Exception {
        OAuth2AccessToken auth2AccessToken = restTemplate.getTokenWithClientCredentialsGrantType();
        if (null == auth2AccessToken) {
            return "获取token失败，请检查配置！";
        }
        return auth2AccessToken.getTokenType() + " " + auth2AccessToken.getValue();
    }
}
